"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _intersection = _interopRequireDefault(require("lodash/intersection"));

var _apolloServer = require("apollo-server");

var _graphql = require("graphql");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable class-methods-use-this, no-underscore-dangle */
class IsAuthenticatedDirective extends _apolloServer.SchemaDirectiveVisitor {
  // static getDirectiveDeclaration(directiveName = 'isAuthenticated') {
  //   return new GraphQLDirective({
  //     name: directiveName,
  //     locations: [DirectiveLocation.FIELD_DEFINITION],
  //   });
  // }
  visitObject(type) {
    const fields = type.getFields();
    Object.keys(fields).forEach(fieldName => {
      const field = fields[fieldName];

      if (!field.isAuthenticatedConfig) {
        field.isAuthenticatedConfig = {};
      }

      field.isAuthenticatedConfig.typeRoles = this.args.roles;
      this.wrapField(field);
    });
  }

  visitFieldDefinition(field) {
    if (!field.isAuthenticatedConfig) {
      field.isAuthenticatedConfig = {}; // eslint-disable-line no-param-reassign
    }

    field.isAuthenticatedConfig.roles = this.args.roles; // eslint-disable-line no-param-reassign

    this.wrapField(field);
  }

  wrapField(field) {
    const {
      isAuthenticatedConfig
    } = field;

    if (isAuthenticatedConfig.isWrapped) {
      return;
    }

    isAuthenticatedConfig.isWrapped = true;
    const {
      resolve = _graphql.defaultFieldResolver
    } = field;
    Object.assign(field, {
      resolve: async (root, args, context, info) => {
        if (!context.user) {
          throw new _apolloServer.AuthenticationError('Not authenticated!');
        }

        const allowedRoles = isAuthenticatedConfig.roles || isAuthenticatedConfig.typeRoles;

        if (Array.isArray(allowedRoles) && allowedRoles.length && !(0, _intersection.default)(allowedRoles, context.user.roles || []).length) {
          throw new _apolloServer.ForbiddenError(`Restricted access! User has to have one of the allowed rolles: ${allowedRoles.join(',')}!`);
        }

        return resolve.call(this, root, args, context, info);
      }
    });
  }

}

var _default = IsAuthenticatedDirective;
exports.default = _default;