"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _crypto = _interopRequireDefault(require("crypto"));

var _easevalidation = require("easevalidation");

var _oors = require("oors");

var _upperFirst = _interopRequireDefault(require("lodash/upperFirst"));

var _omitBy = _interopRequireDefault(require("lodash/omitBy"));

var _isNil = _interopRequireDefault(require("lodash/isNil"));

var _camelCase = _interopRequireDefault(require("lodash/camelCase"));

var _ms = _interopRequireDefault(require("ms"));

var _moment = _interopRequireDefault(require("moment"));

var _validators = require("oors-express/build/validators");

var _withSoftDelete = _interopRequireDefault(require("oors-mongodb/build/decorators/withSoftDelete"));

var _jsonwebtoken = _interopRequireDefault(require("jsonwebtoken"));

var _boom = _interopRequireDefault(require("boom"));

var gqlMiddlewares = _interopRequireWildcard(require("./graphql/middlewares"));

var _jwt = _interopRequireDefault(require("./middlewares/jwt"));

var _passportInitialize = _interopRequireDefault(require("./middlewares/passportInitialize"));

var _passportSession = _interopRequireDefault(require("./middlewares/passportSession"));

var _passport = _interopRequireDefault(require("./libs/passport"));

var _mockUser = _interopRequireDefault(require("./middlewares/mockUser"));

var _PermissionsManager = _interopRequireDefault(require("./libs/PermissionsManager"));

var _user = require("./constants/user");

var _userFromJwt = _interopRequireDefault(require("./middlewares/userFromJwt"));

var _helpers = require("./libs/helpers");

var _FailedLogin = _interopRequireDefault(require("./errors/FailedLogin"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class UserModule extends _oors.Module {
  constructor(...args) {
    super(...args);
    this.name = 'oors.user';

    this.createToken = (user, options = {}) => _jsonwebtoken.default.sign(this.serializeUser(user), this.getConfig('jwtSecret'), _objectSpread({}, this.getConfig('jwtConfig'), {}, options));

    this.serializeUser = data => ({
      id: data._id.toString(),
      username: data.username,
      accountId: data.accountId.toString(),
      roles: data.roles
    });

    this.login = async ({
      username,
      password
    }) => {
      const {
        User
      } = this.get('repositories');
      const user = await User.findOneByUsername(username);

      if (!user) {
        throw new _FailedLogin.default('User not found!');
      }

      await this.canLogin(user);

      if (this.getConfig('lockable.enabled') && user.failedLoginAttempts && user.failedLoginAttempts > this.getConfig('lockable.maxFailedAttempts')) {
        throw new _FailedLogin.default('User is locked because of failed login attempts!');
      }

      const isValidPassword = await this.checkPassword({
        user,
        password
      });

      if (!isValidPassword) {
        if (this.getConfig('lockable.enabled')) {
          await User.updateOne({
            query: {
              _id: user._id
            },
            update: {
              $inc: {
                failedLoginAttempts: 1
              }
            }
          });
        }

        throw new _FailedLogin.default('Incorrect password!');
      }

      return this.tryLogin(user);
    };

    this.loginWithToken = async token => {
      const {
        User
      } = this.get('repositories');
      const user = await User.findOne({
        query: {
          'login.token': token,
          'login.createdAt': {
            $gte: (0, _moment.default)().subtract((0, _ms.default)(this.getConfig('loginTokenExpiresIn'))).toDate()
          }
        }
      });

      if (!user) {
        throw new _FailedLogin.default('User not found!');
      }

      await this.canLogin(user);
      return this.tryLogin(user);
    };

    this.canLogin = async user => {
      if (user.isDeleted) {
        throw new _FailedLogin.default('User not found!');
      }

      if (!user.isActive) {
        throw new _FailedLogin.default('User is not active!');
      }

      const account = user.account || (await this.get('repositories').Account.findById(user.accountId));

      if (!account) {
        throw new _FailedLogin.default('Unable to find user account!');
      }

      if (!account.confirmation.isCompleted) {
        throw new _FailedLogin.default('Account is not confirmed!');
      }

      if (!account.isActive) {
        throw new _FailedLogin.default('Account is not active!');
      }
    };

    this.checkPassword = async ({
      password,
      user
    }) => {
      if (!user.password) {
        return false;
      }

      const hashedPassword = await this.getConfig('hashPassword')(password, user.salt);
      return user.password === hashedPassword;
    };

    this.tryLogin = async user => {
      const {
        User
      } = this.get('repositories');
      const updatedUser = await User.updateOne({
        query: {
          _id: user._id
        },
        update: {
          $set: {
            lastLogin: new Date(),
            resetPassword: {},
            failedLoginAttempts: 0,
            login: {}
          }
        }
      });
      const token = await this.createToken(updatedUser);
      return {
        user: updatedUser,
        token
      };
    };

    this.changePassword = async ({
      userId,
      oldPassword,
      password
    }) => {
      const {
        User
      } = this.get('repositories');
      const user = await User.findById(userId);

      if (!user) {
        throw _boom.default.badRequest('User not found!');
      }

      await this.canLogin(user);

      if (user.password && !this.checkPassword({
        password: oldPassword,
        user
      })) {
        throw _boom.default.badRequest('Invalid password!');
      }

      if (user.password && this.checkPassword({
        password,
        user
      })) {
        throw _boom.default.badRequest("You can't use the same password!");
      }

      return User.updatePassword({
        userId: user._id,
        password: await this.getConfig('hashPassword')(password, user.salt)
      });
    };

    this.recoverPassword = async ({
      password,
      token
    }) => {
      const {
        User
      } = this.get('repositories');
      const user = await User.findOne({
        query: {
          'resetPassword.token': token,
          'resetPassword.resetAt': {
            $gt: (0, _moment.default)().subtract((0, _ms.default)(this.getConfig('resetPasswordTokenExpiresIn'))).toDate()
          }
        }
      });

      if (!user) {
        throw _boom.default.badRequest('Token not found!');
      }

      await this.canLogin(user);
      const hashedPassword = await this.getConfig('hashPassword')(password, user.salt);

      if (hashedPassword === user.password) {
        throw _boom.default.badRequest("You can't use the same password!");
      }

      return User.updatePassword({
        userId: user._id,
        password: hashedPassword
      });
    };

    this.signup = async data => {
      const {
        User,
        Account
      } = this.get('repositories');
      const {
        username,
        email
      } = data;
      const existingUser = await User.findOneByUsernameOrEmail({
        username,
        email
      });

      if (existingUser) {
        if (existingUser.username === username) {
          throw _boom.default.badRequest('Username already taken.');
        }

        if (existingUser.email === email) {
          throw _boom.default.badRequest('Email already taken.');
        }
      }

      const account = await Account.createOne({
        confirmation: {
          token: _crypto.default.randomBytes(20).toString('hex')
        }
      });
      const user = await User.createOne(_objectSpread({
        accountId: account._id
      }, data, {
        isOwner: true
      }));
      this.asyncEmit('signup', {
        user,
        account
      });
      user.account = account;
      return user;
    };

    this.resetPassword = async usernameOrEmail => {
      const {
        User
      } = this.get('repositories');
      const user = await User.findOneByUsername(usernameOrEmail);

      if (!user) {
        throw _boom.default.badRequest('User not found!');
      }

      await this.canLogin(user);
      const updatedUser = await User.resetPassword(user._id);
      this.asyncEmit('resetPassword', updatedUser);
      return updatedUser;
    };

    this.socialLogin = async ({
      req,
      accessToken,
      refreshToken,
      profile
    }, parseProfile = ({
      displayName,
      username
    }) => ({
      name: displayName,
      username
    })) => {
      const {
        User,
        Account
      } = this.get('repositories');
      const email = Array.isArray(profile.emails) && profile.emails.length ? profile.emails[0].value : undefined;
      const matchesProfileId = {
        [`authProviders.${profile.provider}.id`]: profile.id
      };
      const query = email ? {
        $or: [matchesProfileId, {
          email
        }]
      } : matchesProfileId;
      let user = await User.findOne({
        query
      });

      if (!user) {
        const account = await Account.createOne({
          confirmation: {
            isCompleted: true
          }
        });
        user = await User.createOne(_objectSpread({
          accountId: account._id,
          isOwner: true,
          authProviders: {
            [profile.provider]: {
              id: profile.id,
              accessToken,
              refreshToken,
              createdAt: new Date()
            }
          },
          email
        }, parseProfile(profile)));
      } else {
        user = await User.updateOne({
          query: {
            _id: user._id
          },
          update: {
            $set: (0, _omitBy.default)({
              [`authProviders.${profile.provider}.accessToken`]: accessToken,
              [`authProviders.${profile.provider}.refreshToken`]: refreshToken,
              [`authProviders.${profile.provider}.updatedAt`]: new Date()
            }, _isNil.default)
          }
        });
      }

      await this.canLogin(user);

      if (req) {
        await this.saveLogin({
          req,
          user
        });
      }

      return User.updateOne({
        query: {
          _id: user._id
        },
        update: {
          $set: {
            lastLogin: new Date()
          }
        }
      });
    };

    this.saveLogin = async ({
      req,
      user
    }) => {
      if (!this.getConfig('saveLogins')) {
        return;
      }

      await this.get('repositories').Login.createOne({
        userId: user._id,
        ip: req.IP,
        browser: req.useragent.browser,
        os: req.useragent.os,
        platform: req.useragent.platform
      });
    };
  }

  initialize({
    jwtSecret
  }) {
    this.jwtMiddleware = _objectSpread({}, _jwt.default, {
      params: _objectSpread({}, _jwt.default.params, {
        secret: jwtSecret
      })
    });
    this.configureSeeder();
  }

  async setup({
    mockUserMiddlewarePivot,
    jwtMiddlewarePivot,
    mockUserConfig,
    hashPassword
  }) {
    await this.loadDependencies(['oors.mongodb', 'oors.router', 'oors.graphql', 'oors.express']);
    const {
      User,
      Account
    } = this.get('repositories');
    (0, _withSoftDelete.default)()(User);
    (0, _withSoftDelete.default)()(Account);
    User.validators.push(_easevalidation.validators.isProperty('roles', _easevalidation.validators.isArray(_easevalidation.validators.isOneOf(this.getConfig('roles')))));
    this.setupPermissions();
    this.export({
      gqlMiddlewares,
      jwtMiddleware: this.jwtMiddleware,
      hashPassword
    });
    this.configurePassport();
    this.deps['oors.express'].middlewares.insert(jwtMiddlewarePivot, this.jwtMiddleware, _userFromJwt.default);

    if (mockUserConfig.enabled) {
      this.deps['oors.express'].middlewares.insert(mockUserMiddlewarePivot, _objectSpread({}, _mockUser.default, {}, mockUserConfig));
    }

    this.exportProperties(['login', 'canLogin', 'checkPassword', 'tryLogin', 'signup', 'changePassword', 'recoverPassword', 'resetPassword', 'socialLogin', 'createToken', 'findLockedUsers', 'loginWithToken', 'saveLogin']);
  }

  configureSeeder() {
    this.addHook('oors.mongodb', 'configureSeeder', ({
      seeder
    }) => {
      const {
        User,
        Account
      } = this.get('repositories');
      seeder.addResolver('users', User.createOne.bind(User), {
        account: 'accounts'
      });
      seeder.addResolver('accounts', Account.createOne.bind(Account));
    });
  }

  configurePassport() {
    const {
      User
    } = this.get('repositories');
    const {
      passportConfig,
      passportMiddlewarePivot,
      jwtSecret
    } = this.getConfig();

    if (!passportConfig.enabled) {
      return false;
    }

    const passport = (0, _passport.default)({
      jwtSecret,
      User,
      canLogin: this.canLogin,
      login: this.login
    });
    this.deps['oors.express'].middlewares.insert(passportMiddlewarePivot, _passportInitialize.default, _passportSession.default);
    this.export({
      passport
    });
    return passport;
  }

  setupPermissions() {
    this.permissions = new _PermissionsManager.default();
    this.export({
      permissions: this.permissions
    });
    this.getConfig('roles').forEach(role => {
      this.permissions.define(`is${(0, _upperFirst.default)((0, _camelCase.default)(role))}`, user => (user.roles || []).includes(role), {
        description: `User "${role}" role`
      });
    });
    this.deps['oors.graphql'].extendContext(context => {
      Object.assign(context, {
        permissionsManager: this.permissions
      });
    });
  }

  verify({
    token
  }) {
    const {
      User
    } = this.get('repositories'); // @TODO: - check if we need to specify maxAge as an option for jwt.verify

    return new Promise((resolve, reject) => {
      _jsonwebtoken.default.verify(token, this.getConfig('jwtSecret'), async (err, decoded) => {
        if (err) {
          return reject(err);
        }

        try {
          const {
            id
          } = decoded; // @TODO: maybe load these 2 in an aggregate query

          const user = await User.findById(id);
          await this.canLogin(user);
          return resolve(user);
        } catch (error) {
          return reject(error);
        }
      });
    });
  }

}

UserModule.validateConfig = (0, _easevalidation.validate)(_easevalidation.validators.isSchema({
  jwtSecret: [_easevalidation.validators.isRequired(), _easevalidation.validators.isString()],
  jwtConfig: [_easevalidation.validators.isRequired(), _easevalidation.validators.isObject()],
  passportMiddlewarePivot: (0, _validators.isMiddlewarePivot)(),
  mockUserMiddlewarePivot: (0, _validators.isMiddlewarePivot)(),
  jwtMiddlewarePivot: (0, _validators.isMiddlewarePivot)(),
  mockUserConfig: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    path: [_easevalidation.validators.isDefault('/'), _easevalidation.validators.isString()],
    enabled: [_easevalidation.validators.isDefault(false), _easevalidation.validators.isBoolean()],
    params: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isObject()]
  })],
  passportConfig: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    enabled: [_easevalidation.validators.isDefault(false), _easevalidation.validators.isBoolean()]
  })],
  hashPassword: [_easevalidation.validators.isDefault(_helpers.hashPassword), _easevalidation.validators.isFunction()],
  resetPasswordTokenExpiresIn: [_easevalidation.validators.isDefault('1d'), _easevalidation.validators.isAny(_easevalidation.validators.isString(), _easevalidation.validators.isNumber())],
  lockable: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    enabled: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
    maxFailedAttempts: [_easevalidation.validators.isDefault(5), _easevalidation.validators.isNumber()]
  })],
  loginTokenExpiresIn: [_easevalidation.validators.isDefault('1d'), _easevalidation.validators.isAny(_easevalidation.validators.isString(), _easevalidation.validators.isNumber())],
  roles: [_easevalidation.validators.isDefault(_user.roles), _easevalidation.validators.isArray(_easevalidation.validators.isString())],
  saveLogins: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()]
}));
UserModule.defaultConfig = {
  'oors.mongodb': {
    repositories: {
      collectionPrefix: 'user'
    }
  },
  'oors.rad': {
    autoload: {
      services: false
    }
  }
};
var _default = UserModule;
exports.default = _default;