"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _passport = _interopRequireDefault(require("passport"));

var _passportLocal = require("passport-local");

var _passportJwt = require("passport-jwt");

var _FailedLogin = _interopRequireDefault(require("../errors/FailedLogin"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _default = ({
  jwtSecret,
  User,
  canLogin,
  login
}) => {
  _passport.default.use(new _passportJwt.Strategy({
    secretOrKey: jwtSecret,
    jwtFromRequest: _passportJwt.ExtractJwt.fromExtractors([_passportJwt.ExtractJwt.fromAuthHeaderWithScheme('Bearer')]),
    passReqToCallback: true
  }, async (req, id, done) => {
    try {
      const user = await User.findById(id);
      await canLogin(user);
      return done(null, user);
    } catch (error) {
      return done(error, false);
    }
  }));

  _passport.default.use(new _passportLocal.Strategy({
    passReqToCallback: true
  }, async (req, username, password, done) => {
    try {
      const {
        user
      } = await login({
        username,
        password
      });
      return done(null, user);
    } catch (error) {
      if (error instanceof _FailedLogin.default) {
        return done(null, false, error);
      }

      return done(error);
    }
  }));

  _passport.default.serializeUser((user, cb) => {
    cb(null, user._id.toString());
  });

  _passport.default.deserializeUser((req, id, cb) => {
    User.findById(id).then(user => cb(null, user), cb);
  });

  return _passport.default;
};

exports.default = _default;