"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _easevalidation = require("easevalidation");

var _oorsMongodb = require("oors-mongodb");

class Account extends _oorsMongodb.Repository {
  async confirm(token) {
    const account = await this.findOne({
      query: {
        'confirmation.token': token
      }
    });

    if (!account) {
      throw new Error('Account not found!');
    }

    if (account.confirmation.isCompleted) {
      throw new Error('Account is already confirmed!');
    }

    return this.updateOne({
      query: {
        _id: account._id
      },
      update: {
        $set: {
          'confirmation.isCompleted': true,
          'confirmation.confirmedAt': new Date()
        }
      }
    });
  }

}

Account.schema = {
  isActive: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
  confirmation: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    isCompleted: [_easevalidation.validators.isDefault(false), _easevalidation.validators.isBoolean()],
    token: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isString()),
    confirmedAt: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isDate())
  })]
};
Account.relations = {
  users: {
    repositoryName: 'oors.user.User',
    localField: '_id',
    foreignField: 'accountId',
    type: 'many'
  }
};
var _default = Account;
exports.default = _default;