"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _easevalidation = require("easevalidation");

var _bcryptjs = _interopRequireDefault(require("bcryptjs"));

var _crypto = _interopRequireDefault(require("crypto"));

var _oorsMongodb = require("oors-mongodb");

var _isObjectId = _interopRequireDefault(require("oors-mongodb/build/libs/isObjectId"));

var _user = require("../constants/user");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

class User extends _oorsMongodb.Repository {
  constructor(...args) {
    super(...args);

    this.getLockedQuery = () => ({
      failedLoginAttempts: {
        $gt: this.module.getConfig('lockable.maxFailedAttempts')
      }
    });
  }

  findOneByUsername(username) {
    return this.findOne({
      query: {
        $or: [{
          username
        }, {
          email: username
        }]
      }
    });
  }

  findOneByUsernameOrEmail({
    username,
    email
  }) {
    return this.findOne({
      query: {
        $or: [{
          username
        }, {
          email
        }]
      }
    });
  }

  updatePassword({
    userId,
    password
  }) {
    return this.updateOne({
      query: {
        _id: userId
      },
      update: {
        $set: {
          resetPassword: {},
          password,
          failedLoginAttempts: 0
        }
      }
    });
  }

  async resetPassword(userId) {
    return this.updateOne({
      query: {
        _id: userId
      },
      update: {
        $set: {
          resetPassword: {
            token: _crypto.default.randomBytes(20).toString('hex'),
            resetAt: new Date()
          }
        }
      }
    });
  }

  createLoginToken(userId) {
    return this.updateOne({
      query: {
        _id: userId
      },
      update: {
        $set: {
          login: {
            token: _crypto.default.randomBytes(20).toString('hex'),
            createdAt: new Date()
          }
        }
      }
    });
  }

  async parse(data) {
    const _ref = await _oorsMongodb.Repository.prototype.parse.call(this, data),
          {
      hashPassword: doHashPassword
    } = _ref,
          finalData = _objectWithoutProperties(_ref, ["hashPassword"]);

    if (!finalData.salt) {
      finalData.salt = _bcryptjs.default.genSaltSync(10);
    }

    const shouldHashPassword = (!finalData[this.id] || doHashPassword) && !!finalData.password;

    if (shouldHashPassword) {
      finalData.password = await this.module.getConfig('hashPassword')(finalData.password, finalData.salt);
    }

    return finalData;
  }

}

User.schema = {
  accountId: [_easevalidation.validators.isRequired(), (0, _isObjectId.default)()],
  username: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isString()),
  name: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isString()),
  email: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isEvery(_easevalidation.validators.isString(), _easevalidation.validators.isEmail())),
  password: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isEvery(_easevalidation.validators.isString(), _easevalidation.validators.isLength({
    min: 5
  }))),
  salt: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isString()),
  isActive: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
  isOwner: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
  roles: [_easevalidation.validators.isDefault(_user.defaultRoles), _easevalidation.validators.isArray(_easevalidation.validators.isString())],
  resetPassword: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    token: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isString()),
    resetAt: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isDate())
  })],
  lastLogin: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isDate()),
  authProviders: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isObject({
    validator: _easevalidation.validators.isSchema({
      id: [_easevalidation.validators.isRequired(), _easevalidation.validators.isString()],
      accessToken: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isString()),
      refreshToken: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isString()),
      createdAt: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isDate()),
      updatedAt: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isDate())
    })
  })],
  failedLoginAttempts: [_easevalidation.validators.isDefault(0), _easevalidation.validators.isNumber()],
  login: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    token: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isString()),
    createdAt: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isDate())
  })]
};
User.relations = {
  account: {
    repositoryName: 'oors.user.Account',
    localField: 'accountId',
    foreignField: '_id',
    type: 'one'
  },
  logins: {
    repositoryName: 'oors.user.Login',
    localField: '_id',
    foreignField: 'userId',
    type: 'many'
  }
};
var _default = User;
exports.default = _default;